"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const amqp = require("amqplib");
const events_1 = require("events");
const logger_1 = require("./logger");
const replyQueue_1 = require("./replyQueue");
const delayQueue_1 = require("./delayQueue");
const queue_1 = require("./queue");
const exchange_1 = require("./exchange");
const assert = require("assert");
class Rabbit extends events_1.EventEmitter {
    constructor(url, { prefetch = 1, replyPattern = true, logger = null, prefix = '', scheduledPublish = false, socketOptions = {} } = {}) {
        super();
        this.url = url;
        this.queues = {};
        this.connecting = false;
        assert(url, 'Url is required!');
        this.prefetch = prefetch;
        this.replyPattern = replyPattern;
        this.prefix = prefix;
        this.scheduledPublish = scheduledPublish;
        this.socketOptions = socketOptions;
        logger_1.init(logger);
        this.reconnect();
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.connecting) {
                return;
            }
            this.connecting = true;
            let connection = yield amqp.connect(this.url, this.socketOptions);
            let channel = yield this.createChannel(connection);
            yield this.initChannel(channel);
        });
    }
    reconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            this.connected = this.connect();
            this.connected.catch(error => this.emitDisconnected(error));
            yield this.connected;
        });
    }
    emitDisconnected(error) {
        this.connecting = false;
        this.emit('disconnected', error);
    }
    createChannel(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            this.connection = connection;
            this.connection.once('close', error => this.emitDisconnected(error));
            this.connection.on('error', error => this.emitDisconnected(error));
            return connection.createConfirmChannel();
        });
    }
    initChannel(channel) {
        return __awaiter(this, void 0, void 0, function* () {
            this.channel = channel;
            this.channel.prefetch(this.prefetch);
            this.channel.on('close', error => this.emitDisconnected(error));
            if (this.replyPattern) {
                yield replyQueue_1.createReplyQueue(this.channel);
            }
            if (this.scheduledPublish) {
                yield delayQueue_1.createDelayQueueReply(this.channel, this.updateName('delay'));
            }
            this.emit('connected');
            this.connecting = false;
        });
    }
    updateName(name, prefix = this.prefix) {
        if (prefix && prefix.length > 0) {
            if (!name.startsWith(`${prefix}_`)) {
                name = `${prefix}_${name}`;
            }
        }
        return name;
    }
    createQueue(name, options = {}, handler) {
        return __awaiter(this, void 0, void 0, function* () {
            name = this.updateName(name, options.prefix);
            yield this.connected;
            const queue = new queue_1.default(this.channel, name, options);
            this.queues[name] = queue;
            yield queue.created;
            logger_1.getLogger().debug(`created queue ${name}`);
            if (handler) {
                yield queue.subscribe(handler);
            }
            return queue;
        });
    }
    destroyQueue(name, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            name = this.updateName(name, prefix);
            yield this.connected;
            yield queue_1.default.destroy(this.channel, name);
        });
    }
    subscribe(name, handler, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            name = this.updateName(name, prefix);
            yield this.connected;
            yield this.queues[name].subscribe(handler);
        });
    }
    unsubscribe(name, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            name = this.updateName(name, prefix);
            yield this.connected;
            yield this.queues[name].unsubscribe();
        });
    }
    publish(name, obj, headers, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            name = this.updateName(name, prefix);
            yield this.connected;
            yield queue_1.default.publish(obj, headers, this.channel, name, this.queues[name]);
        });
    }
    publishWithDelay(name, obj, headers, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.scheduledPublish) {
                throw new Error('scheduledPublish is not enabled');
            }
            name = this.updateName(name, prefix);
            yield this.connected;
            yield delayQueue_1.publishWithDelay(this.updateName('delay'), obj, headers, this.channel, name);
        });
    }
    getReply(name, obj, headers, prefix, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            name = this.updateName(name, prefix);
            yield this.connected;
            return yield queue_1.default.getReply(obj, headers, this.channel, name, this.queues[name], timeout);
        });
    }
    getTopicReply(topicName, content, headers, prefix, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            topicName = this.updateName(topicName, prefix);
            yield this.connected;
            return yield exchange_1.default.getReply(this.channel, 'amq.topic', topicName, content, headers, timeout);
        });
    }
    publishExchange(exchange, routingKey, content, headers, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            routingKey = this.updateName(routingKey, prefix);
            yield this.connected;
            yield exchange_1.default.publish(this.channel, exchange, routingKey, content, headers);
        });
    }
    publishTopic(topicName, content, headers = {}, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            topicName = this.updateName(topicName, prefix);
            yield this.connected;
            yield exchange_1.default.publish(this.channel, 'amq.topic', topicName, content, headers);
        });
    }
    bindToExchange(queueName, exchange, routingKey, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            queueName = this.updateName(queueName, prefix);
            yield this.connected;
            yield queue_1.default.bindToExchange(exchange, routingKey, this.channel, queueName, this.queues[queueName]);
        });
    }
    unbindFromExchange(queueName, exchange, topicName, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            queueName = this.updateName(queueName, prefix);
            yield this.connected;
            yield queue_1.default.unbindFromExchange(exchange, topicName, this.channel, queueName, this.queues[queueName]);
        });
    }
    bindToTopic(queueName, topicName, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            queueName = this.updateName(queueName, prefix);
            yield this.bindToExchange(queueName, 'amq.topic', topicName, prefix);
        });
    }
    unbindFromTopic(queueName, topicName, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            queueName = this.updateName(queueName, prefix);
            yield this.unbindFromExchange(queueName, 'amq.topic', topicName, prefix);
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.connection.close();
        });
    }
}
Rabbit.STOP_PROPAGATION = queue_1.default.STOP_PROPAGATION;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Rabbit;
//# sourceMappingURL=rabbit.js.map